function [mp,mpj] = aconduct(sigma,mu,h,d,R,omega,orient,type)
%ACONDUCT compute the apparent conductivity
%   compute the quadrature component of the ratio H_S/H_P
%   between the secondary and primary field in a layered
%   model with the scale factor 4/(mu0*omega*r^2)
%   [mp,mpj]=aconduct(sigma,mu,h,d,R,omega,orient,jtype)
%   where
%      sigma  - array of conductivities per layer (S/m)
%      mu     - array of magnetic permeabilities per layer (H/m)
%      h      - array of heights above ground (m)
%      d      - array of layers thickness (m)
%      R      - array of intercoil (IC) spacings (m)
%      omega  - array of angular frequencies (rad/s) (=2*pi*f, f in Hz)
%      orient - Instrument orientation: 'vertical', 'horizontal', 
%				`	'perpendicular', 'vh', 'vp'
%      jtype  - Jacobian type: 'sigma', 'mu', 'both'
%   and
%      mp = [mv  - Predicted readings with coils vertically oriented
%            mh] - Predicted readings with coils horizontally oriented
%      mp is either mv or mh for a single orientation
%      mp is further decomposed by IC-spacing, height, and frequency:
%         mp = [mp(R_1), mp(R_2), ...]
%         mp(R_i) = [mp(h_1), mp(h_2), ...]
%         mp(h_i) = [mp(omega_1), mp(omega_2), ...]
%
%      mpj is either mpjsigma, mpjmu, or [mpjsigma,mpjmu], where
%      mpjsigma = Jacobian matrix w.r.to sigma_j (same orientation)
%      mpjmu    = Jacobian matrix w.r.to mu_j (same orientation)

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised January 26, 2023

ver = 0;
hor = 0;
per = 0;
perp = 0;
switch lower(orient(1:2))
	case 've'
		ver = 1;
	case 'ho'
		hor = 1;
	case 'pe'
		per = 1;
	case 'vh'
		ver = 1;
		hor = 1;
	case 'vp'
		ver = 1;
		perp = 1;
	otherwise
		error('Unknown orientation.')
end
dR = 0.1;	% diff R for vertical/perpendicular coils

compjack = false;
compsigma = false;
compmu = false;
if nargout > 1
	compjack = true;
	if nargin>7
		switch lower(type(1))
			case 's'
				compsigma = true;
			case 'm'
				compmu = true;
			case 'b'
				compsigma = true;
				compmu = true;
			otherwise
				error('The type parameter is wrong.')
		end
	else
		error('The type parameter is missing.')
	end
end

sigma = sigma(:);
mu = mu(:);
R = R(:);
Rp = R+dR;
omega = omega(:);
n = size(sigma,1);
m = size(h,1);
nR = size(R,1);
nfreq = size(omega,1);
nhf = m*nfreq;
mu0 = pi*4e-7;

if nargin<7 || isempty(orient), orient = 'vh'; end
if nargin<6 || isempty(omega) || isempty(R) || isempty(h) ...
		|| isempty(mu) || isempty(sigma)
	error('Some input parameter are not assigned')
end

[w0,w1] = hankelwts;	% get weights for Hankel transform
y = hankelpts(1);	% get nodes for Hankel transform
%q = size(y,1);

if ver
	T0 = zeros(m,nfreq);
	mv = zeros(nR*nhf,1);
	mvjs = zeros(nR*nhf,n);
	mvjm = zeros(nR*nhf,n);
end
if hor 
	T2 = zeros(m,nfreq);
	mh = zeros(nR*nhf,1);
	mhjs = zeros(nR*nhf,n);
	mhjm = zeros(nR*nhf,n);
end
if per
	T2 = zeros(m,nfreq);
	mo = zeros(nR*nhf,1);
	mojs = zeros(nR*nhf,n);
	mojm = zeros(nR*nhf,n);
end
if perp
	%T0p = zeros(m,nfreq);
	T2p = zeros(m,nfreq);
	mop = zeros(nR*nhf,1);
	mojsp = zeros(nR*nhf,n);
	mojmp = zeros(nR*nhf,n);
end

for k = 1:nR
	yh = y/R(k);
	yhp = y/Rp(k);
	if compjack
		if compsigma && compmu
			if ver || hor || per
				[rf,rfjsigma,rfjmu] = reflfact(yh,sigma,mu,d,omega);
				rf = imag(rf);
				rfjsigma = imag(rfjsigma);
				rfjmu = imag(rfjmu);
			end
			if perp
				[rfp,rfjsigmap,rfjmup] = reflfact(yhp,sigma,mu,d,omega);
				rfp = imag(rfp);
				rfjsigmap = imag(rfjsigmap);
				rfjmup = imag(rfjmup);
			end
		elseif compsigma
			if ver || hor || per
				[rf,rfjsigma] = reflfact(yh,sigma,mu,d,omega,'sigma');
				rf = imag(rf);
				rfjsigma = imag(rfjsigma);
			end
			if perp
				[rfp,rfjsigmap] = reflfact(yhp,sigma,mu,d,omega,'sigma');
				rfp = imag(rfp);
				rfjsigmap = imag(rfjsigmap);
			end
		elseif compmu
			if ver || hor || per
				[rf,rfjmu] = reflfact(yh,sigma,mu,d,omega,'mu');
				rf = imag(rf);
				rfjmu = imag(rfjmu);
			end
			if perp
				[rfp,rfjmup] = reflfact(yhp,sigma,mu,d,omega,'mu');
				rfp = imag(rfp);
				rfjmup = imag(rfjmup);
			end
		else
			error('Problems..')
		end

		% the definition of Hankel transf. used in hankelpts.m
		% does not include lambda in the kernel.
		if compsigma
			if ver || hor || per
				rfjsigma = rfjsigma.*repmat(yh,[1,n,nfreq]);
			end
			if perp
				rfjsigmap = rfjsigmap.*repmat(yhp,[1,n,nfreq]);
			end
		end
		if compmu
			if ver || hor || per
				rfjmu = rfjmu.*repmat(yh,[1,n,nfreq]);
			end
			if perp
				rfjmup = rfjmup.*repmat(yhp,[1,n,nfreq]);
			end
		end
	else
		if ver || hor || per
			rf = reflfact(yh,sigma,mu,d,omega);
			rf = imag(rf);
		end
		if perp
			rfp = reflfact(yhp,sigma,mu,d,omega);
			rfp = imag(rfp);
		end
	end

	if ver || hor || per
		rf = rf.*repmat(yh,1,nfreq);
	end
	if perp
		rfp = rfp.*repmat(yhp,1,nfreq);
	end

	for i = 1:m
		if ver || hor || per
			f = rf.*repmat(exp(-2*yh*h(i)),1,nfreq);
			% this computes H_0(r) for r=R(k)
			T0(i,:) = w0'*(f.*repmat(yh,1,nfreq))/R(k);
			% this computes H_1(r) for r=R(k)
			T2(i,:) = w1'*f/R(k);
		end
		if perp
			fp = rfp.*repmat(exp(-2*yhp*h(i)),1,nfreq);
			%T0p(i,:) = w0'*(fp.*repmat(yhp,1,nfreq))/Rp(k);
			T2p(i,:) = w1'*fp/Rp(k);
		end
	end

	f1 = 4./(mu0*omega');
	if ver
		mvt = -R(k)*repmat(f1,m,1).*T0;
		mv((k-1)*nhf+1:k*nhf) = mvt.';
	end
	if hor
		mht = -repmat(f1,m,1).*T2;
		mh((k-1)*nhf+1:k*nhf) = mht.';
	end
	if per
		mot = -repmat(f1,m,1).*T2;
		mo((k-1)*nhf+1:k*nhf) = mot.';
	end
	if perp
		motp = -repmat(f1,m,1).*T2p;
		mop((k-1)*nhf+1:k*nhf) = motp.';
	end

	if compjack
		if compsigma
			if ver
				T0j = zeros(m,n,nfreq);
				for i = 1:m
					fj = rfjsigma.*repmat(exp(-2*yh*h(i)),...
						[1,n,nfreq]);
					T0j(i,:,:) = sum(fj.*repmat(w0.*yh,...
						[1,n,nfreq]))/R(k);
				end
			end
			if hor || per
				T2j = zeros(m,n,nfreq);
				for i = 1:m
					fj = rfjsigma.*repmat(exp(-2*yh*h(i)),...
						[1,n,nfreq]);
					T2j(i,:,:) = sum(fj.*repmat(w1,...
						[1,n,nfreq]))/R(k);
				end
			end
			if perp
				%T0jp = zeros(m,n,nfreq);
				T2jp = zeros(m,n,nfreq);
				for i = 1:m
					fjp = rfjsigmap.*repmat(exp(-2*yhp*h(i)),...
						[1,n,nfreq]);
					%T0jp(i,:,:) = sum(fjp.*repmat(w0.*yhp,...
					%	[1,n,nfreq]))/Rp(k);
					T2jp(i,:,:) = sum(fjp.*repmat(w1,...
						[1,n,nfreq]))/Rp(k);
				end
			end

			if ver
				mvjt = -R(k)*permute(repmat(f1,[m,1,n]),[1 3 2]).*T0j;
				mvjt = reshape(permute(mvjt,[3 1 2]),nfreq*m,n);
				mvjs((k-1)*nhf+1:k*nhf,:) = mvjt;
			end
			if hor
				mhjt = -permute(repmat(f1,[m,1,n]),[1 3 2]).*T2j;
				mhjt = reshape(permute(mhjt,[3 1 2]),nfreq*m,n);
				mhjs((k-1)*nhf+1:k*nhf,:) = mhjt;
			end
			if per
				mojt = -permute(repmat(f1,[m,1,n]),[1 3 2]).*T2j;
				mojt = reshape(permute(mojt,[3 1 2]),nfreq*m,n);
				mojs((k-1)*nhf+1:k*nhf,:) = mojt;
			end
			if perp
				mojtp = -permute(repmat(f1,[m,1,n]),[1 3 2]).*T2jp;
				mojtp = reshape(permute(mojtp,[3 1 2]),nfreq*m,n);
				mojsp((k-1)*nhf+1:k*nhf,:) = mojtp;
			end
		end
		if compmu
			if ver
				T0j = zeros(m,n,nfreq);
				for i = 1:m
					fj = rfjmu.*repmat(exp(-2*yh*h(i)),[1,n,nfreq]);
					T0j(i,:,:) = sum(fj.*repmat(w0.*yh,...
						[1,n,nfreq]))/R(k);
				end
			end
			if hor || per
				T2j = zeros(m,n,nfreq);
				for i = 1:m
					fj = rfjmu.*repmat(exp(-2*yh*h(i)),[1,n,nfreq]);
					T2j(i,:,:) = sum(fj.*repmat(w1,...
						[1,n,nfreq]))/R(k);
				end
			end
			if perp
				%T0jp = zeros(m,n,nfreq);
				T2jp = zeros(m,n,nfreq);
				for i = 1:m
					fjp = rfjmup.*repmat(exp(-2*yhp*h(i)),[1,n,nfreq]);
					%T0jp(i,:,:) = sum(fjp.*repmat(w0.*yhp,...
					%	[1,n,nfreq]))/Rp(k);
					T2jp(i,:,:) = sum(fjp.*repmat(w1,...
						[1,n,nfreq]))/Rp(k);
				end
			end

			if ver
				mvjt = -R(k)*permute(repmat(f1,[m,1,n]),[1 3 2]).*T0j;
				mvjt = reshape(permute(mvjt,[3 1 2]),nfreq*m,n);
				mvjm((k-1)*nhf+1:k*nhf,:) = mvjt;
			end
			if hor
				mhjt = -permute(repmat(f1,[m,1,n]),[1 3 2]).*T2j;
				mhjt = reshape(permute(mhjt,[3 1 2]),nfreq*m,n);
				mhjm((k-1)*nhf+1:k*nhf,:) = mhjt;
			end
			if per
				mojt = -permute(repmat(f1,[m,1,n]),[1 3 2]).*T2j;
				mojt = reshape(permute(mojt,[3 1 2]),nfreq*m,n);
				mojm((k-1)*nhf+1:k*nhf,:) = mojt;
			end
			if perp
				mojtp = -permute(repmat(f1,[m,1,n]),[1 3 2]).*T2jp;
				mojtp = reshape(permute(mojtp,[3 1 2]),nfreq*m,n);
				mojmp((k-1)*nhf+1:k*nhf,:) = mojtp;
			end
		end
	end
end

switch lower(orient(1:2))
	case 've'
		mp = mv;
	case 'ho'
		mp = mh;
	case 'pe'
		mp = mo;
	case 'vh'
		mp = [mv;mh];
	case 'vp'
		mp = [mv;mop];
end

if compjack
	if compsigma
		switch lower(orient(1:2))
			case 've'
				mpjsigma = mvjs;
			case 'ho'
				mpjsigma = mhjs;
			case 'pe'
				mpjsigma = mojs;
			case 'vh'
				mpjsigma = [mvjs;mhjs];
			case 'vp'
				mpjsigma = [mvjs;mojsp];
		end
	end
	if compmu
		switch lower(orient(1:2))
			case 've'
				mpjmu = mvjm;
			case 'ho'
				mpjmu = mhjm;
			case 'pe'
				mpjmu = mojm;
			case 'vh'
				mpjmu = [mvjm;mhjm];
			case 'vp'
				mpjmu = [mvjm;mojmp];
		end
	end
end

if compsigma && compmu
	mpj = [mpjsigma,mpjmu];
elseif compsigma
	mpj = mpjsigma;
elseif compmu
	mpj = mpjmu;
end

