function [CRs,CRm,z] = cumulativeresp(sigma,mu,z,lastzplot,h,d,R,omega,HVorient,modeltype,fff)
%CUMULATIVERESP compute the cumulative response
%   cumulativeresp(sigma,mu,z,lastz,h,d,R,omega,HVorient,modeltype)
%   where
%       sigma       - array of conductivities per layer (S/m)
%       mu          - array of magnetic permeabilities per layer (H/m)
%       z           - array of depths (m)
%	lastz	    - max z displayed on graphs
%       h           - array of heights above ground (m)
%       d           - array of layers thickness (m)
%       R           - array of intercoil (IC) spacings (m)
%       omega       - array of angular frequencies (rad/s) (=2*pi*f, f in Hz)
%       HVorient    - instrument orientation: 'vertical', 'horizontal', 
%				'perpendicular', 'vh', 'vp'
%       modeltype   - kind of instruments readings: 1 aconduct, 2 quadracomp,
%				3 inphase, 4 complexquad, 5 complexacon
%   and
%       Crs - cumulative response for the electrical conductivity
%       Crm - cumulative response for the magnetic permeability
%       z   - array of depths (m)
%
%   G.P. Deidda, P. Díaz de Alba, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised January 26, 2023

if nargin<11, fff = 0; end
legendavail = 1;
pos = 'southeast';
freq = omega/(2*pi);
lastz = z(end);
n = length(z);
vv = find(z<lastz);
nlz = min(vv(end),n-1);

switch modeltype		% define the model to invert
	case 1
		func = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 2
		func = @(sigma,mu) quadracomp(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) quadracomp(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 3
		func = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 4
		func = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 5
		funcinphase = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcacond = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcinphasemu = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'mu');
		funcacondmu = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'mu');
	%case 5, func = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'sigma');
end

if modeltype == 5
	[~,Js1] = funcinphase(sigma,mu);
	[~,Js2] = funcacond(sigma,mu);
	A = Js1 + Js2*1i;
	[~,Jm1] = funcinphasemu(sigma,mu);
	[~,Jm2] = funcacondmu(sigma,mu);
	B = Jm1 + Jm2*1i;
else
	[~,A] = func(sigma,mu);
	[~,B] = funcmu(sigma,mu);
end
xlab = 'Depth (m)';
legendsens = strcat('\rho=',strtrim(cellstr(num2str(R(:)))),',freq=',strtrim(cellstr(num2str(freq(:)))),',h=',strtrim(cellstr(num2str(h(:)))));
if modeltype == 1
	A = 1e3.*A/(lastz/(n-1));
	B = 1e-3.*B/(lastz/(n-1));
	CRs = zeros(size(A,1),nlz);
	CRm = zeros(size(B,1),nlz);
	for i=1:size(A,1)
		for j=1:nlz
			fs = A(i,j:end-1);
			fs = abs(fs);
			hs = (z(end-1)-z(j))/length(fs);
			Is = fdemsimp(fs,hs);
			CRs(i,j) = Is;
		end
		%CRs(i,:) = CRs(i,:)./max(CRs(i,:));
	end
	for i=1:size(B,1)
		for j=1:nlz
			fm = B(i,j:end-1);
			fm = abs(fm);
			hm = (z(end-1)-z(j))/length(fm);
			Im = fdemsimp(fm,hm);
			CRm(i,j) = Im;
		end
		%CRm(i,:) = CRm(i,:)./max(CRm(i,:));
	end
	ylab1 = 'Cumulative [(mS/m)/(S/m)]';
	ylab2 = '';
	ylab3 = 'Cumulative [(mS/m)/(\muH/m)]';
	ylab4 = '';
elseif modeltype == 2
	A = 1e6.*A/(lastz/(n-1));
	CRs = zeros(size(A,1),nlz);
	CRm = zeros(size(B,1),nlz);
	for i=1:size(A,1)
		for j=1:nlz
			fs = A(i,j:end-1);
			fs = abs(fs);
			hs = (z(end-1)-z(j))/length(fs);
			Is = fdemsimp(fs,hs);
			CRs(i,j) = Is;
		end
		%CRs(i,:) = CRs(i,:)./max(CRs(i,:));
	end
	for i=1:size(B,1)
		for j=1:nlz
			fm = B(i,j:end-1);
			fm = abs(fm);
			hm = (z(end-1)-z(j))/length(fm);
			Im = fdemsimp(fm,hm);
			CRm(i,j) = Im;
		end
		%CRm(i,:) = CRm(i,:)./max(CRm(i,:));
	end
	ylab1 = 'Cumulative [ppm/(S/m)]';
	ylab2 = '';
	ylab3 = 'Cumulative [ppm/(\muH/m)]';
	ylab4 = '';
elseif modeltype == 3
	A = 1e6.*A/(lastz/(n-1));
	CRs = zeros(size(A,1),nlz);
	CRm = zeros(size(B,1),nlz);
	for i=1:size(A,1)
		for j=1:nlz
			fs = A(i,j:end-1);
			fs = abs(fs);
			hs = (z(end-1)-z(j))/length(fs);
			Is = fdemsimp(fs,hs);
			CRs(i,j) = Is;
		end
		%CRs(i,:) = CRs(i,:)./max(CRs(i,:));
	end
	for i=1:size(B,1)
		for j=1:nlz
			fm = B(i,j:end-1);
			fm = abs(fm);
			hm = (z(end-1)-z(j))/length(fm);
			Im = fdemsimp(fm,hm);
			CRm(i,j) = Im;
		end
		%CRm(i,:) = CRm(i,:)./max(CRm(i,:));
	end
	ylab1 = 'Cumulative [ppm/(S/m)]';
	ylab2 = '';
	ylab3 = 'Cumulative [ppm/(\muH/m)]';
	ylab4 = '';
elseif modeltype == 4
	A = 1e6.*A/(lastz/(n-1));
	CRsr = zeros(size(A,1),nlz);
	CRsi = zeros(size(A,1),nlz);
	CRmr = zeros(size(B,1),nlz);
	CRmi = zeros(size(B,1),nlz);
	for i=1:size(A,1)
		for j=1:nlz
			fsr = real(A(i,j:end-1));
			fsr = abs(fsr);
			fsi = imag(A(i,j:end-1));
			fsi = abs(fsi);
			hs = (z(end-1)-z(j))/length(fsr);
			Isr = fdemsimp(fsr,hs);
			Isi = fdemsimp(fsi,hs);
			CRsr(i,j) = Isr;
			CRsi(i,j) = Isi;
		end
		%CRsr(i,:) = CRsr(i,:)./max(CRsr(i,:));
		%CRsi(i,:) = CRsi(i,:)./max(CRsi(i,:));
	end
	CRs = CRsr+1i*CRsi;
	for i=1:size(B,1)
		for j=1:nlz
			fmr = real(B(i,j:end-1));
			fmr = abs(fmr);
			fmi = imag(B(i,j:end-1));
			fmi = abs(fmi);
			hm = (z(end-1)-z(j))/length(fmr);
			Imr = fdemsimp(fmr,hm);
			Imi = fdemsimp(fmi,hm);
			CRmr(i,j) = Imr;
			CRmi(i,j) = Imi;
		end
		%CRmr(i,:) = CRmr(i,:)./max(CRmr(i,:));
		%CRmi(i,:) = CRmi(i,:)./max(CRmi(i,:));
	end
	CRm = CRmr+1i*CRmi;
	ylab1 = 'Cumulative [ppm/(S/m)]';
	ylab2 = 'Cumulative [ppm/(S/m)]';
	ylab3 = 'Cumulative [ppm/(\muH/m)]';
	ylab4 = 'Cumulative [ppm/(\muH/m)]';
elseif modeltype == 5
	A = 1e3.*A/(lastz/(n-1));
	B = 1e-3.*B/(lastz/(n-1));
	CRsr = zeros(size(A,1),nlz);
	CRsi = zeros(size(A,1),nlz);
	CRmr = zeros(size(B,1),nlz);
	CRmi = zeros(size(B,1),nlz);
	for i=1:size(A,1)
		for j=1:nlz
			fsr = real(A(i,j:end-1));
			fsr = abs(fsr);
			fsi = imag(A(i,j:end-1));
			fsi = abs(fsi);
			hs = (z(end-1)-z(j))/length(fsr);
			Isr = fdemsimp(fsr,hs);
			Isi = fdemsimp(fsi,hs);
			CRsr(i,j) = Isr;
			CRsi(i,j) = Isi;
		end
		%CRsr(i,:) = CRsr(i,:)./max(CRsr(i,:));
		%CRsi(i,:) = CRsi(i,:)./max(CRsi(i,:));
	end
	CRs = CRsr+1i*CRsi;
	for i=1:size(B,1)
		for j=1:nlz
			fmr = real(B(i,j:end-1));
			fmr = abs(fmr);
			fmi = imag(B(i,j:end-1));
			fmi = abs(fmi);
			hm = (z(end-1)-z(j))/length(fmr);
			Imr = fdemsimp(fmr,hm);
			Imi = fdemsimp(fmi,hm);
			CRmr(i,j) = Imr;
			CRmi(i,j) = Imi;
		end
		%CRmr(i,:) = CRmr(i,:)./max(CRmr(i,:));
		%CRmi(i,:) = CRmi(i,:)./max(CRmi(i,:));
	end
	CRm = CRmr+1i*CRmi;
	ylab1 = 'Cumulative [ppt/(S/m)]';
	ylab2 = 'Cumulative [(mS/m)/(S/m)]';
	ylab3 = 'Cumulative [ppt/(\muH/m)]';
	ylab4 = 'Cumulative [(mS/m)/(\muH/m)]';
end

if fff
	plotcumulative(fff,CRs,CRm,z,lastzplot,HVorient,xlab,ylab1,ylab2,ylab3,ylab4,legendavail,pos,legendsens,modeltype)
end

CRs = CRs.';
CRm = CRm.';
z = z(1:end-1);

