function [mu,kappa,rho,eta,X,err,fail] = emsolvenlmu(func,y,h,d,opts,textarea,app)
%EMSOLVENLMU reconstruct the magnetic permeability
%   solve non linear model for FDEM with respect to permeability
%   [mu,kappa,rho,eta,X,errors,fail] = emsolvenlmu(func,y,h,d,opts,textarea,app)
%   where
%      func  - signal model function
%      y     - input data set
%      h     - array of heights above ground (m)
%      d     - array of layers thickness (m)
%      opts  - options (struct, every options has a default value)
%              Lind: index of regularization matrix (k-th derivative)
%              orientation: orientation of device
%              show: >0 show information on iterations
%              rowcol: size of grid in the iteration graphical window
%              metk: methods for choosing the regularization parameter
%                    (cell array of strings)
%              taudiscr: tau factor for discrepancy principle
%              truesol: exact solution, if available (to compute errors)
%              ds: standard deviation of noise for discrepancy principle
%              r_in: truncation parameter for noreg
%              textarea - window area where display information (used by app)
%              app  - code called by app
%              the following options are passed to tsvdnewt
%              tau, nmax, damped, dampos, metjac, kbroyden, mnflag, xbar
%   and
%      mu     - solutions (one for each method in opts.metk)
%      kappa  - regularization parameters identified
%      rho    - residuals for each regularized solution
%      eta    - weighted norm for each regularized solution
%      X      - regularized solutions and other info
%      errors - errors with respect to true solution
%      fail   - failures of reg. parameters identification

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised September 16, 2022

if exist('l_corner','file') ~= 2
	error('Please install Regularization Tools.')
end

if (nargin < 6), app.stop = 0; end
if (nargin > 5) && ~isempty(textarea), calledbyapp = 1; else, calledbyapp = 0; textarea = []; end
if nargin < 5 || isempty(opts), opts = struct('empty','empty'); end
if nargin < 4, error('Too few input arguments.'); end

mm = size(y,1);
m = size(h,1);
n = size(d,1)+1;
mu0 = pi*4e-7;

nam = fieldnames(opts);
% rows to keep in model prediction
if ~any(strcmpi('yrows',nam)), yrows = [1:mm]'; else, yrows = opts.yrows; end
% starting vector for permeability mu
if ~any(strcmpi('muinit',nam)), opts.muinit = mu0*1e2*ones(n,1); end
% index of regularization matrix
if ~any(strcmpi('Lind',nam)), opts.Lind = 1; end
% orientation of device
if ~any(strcmpi('orientation',nam))
	if m == mm
		opts.orientation = 'vertical';
	elseif 2*m == mm
		opts.orientation = 'vh';
	else
		error('Size of data is inconsistent.')
	end
end
% method for choosing the regularization parameter
if ~any(strcmpi('metk',nam)), opts.metk = 'corner'; end
%  show intermediate results & graphs
if ~any(strcmpi('show',nam)), opts.show = 0; end
%  show identification of parameters
if ~any(strcmpi('showpar',nam)), opts.showpar = 0; end
%  how many graphs rows/columns in the show window
if ~any(strcmpi('rowcol',nam)), opts.rowcol = [2 4]; end
%  xlim for the graphs in the show window
if ~any(strcmpi('xlim',nam)), opts.xlim = []; end
%  ylim for the graphs in the show window
if ~any(strcmpi('ylim',nam)), opts.ylim = []; end
%  if true solution is available, compute errors
if ~any(strcmpi('truesol',nam)), opts.truesol = []; end
%  standard deviation of noise for discrepancy principle
if ~any(strcmpi('ds',nam)), opts.ds = []; end
%  factor tau for discrepancy principle
if ~any(strcmpi('taudiscr',nam)), opts.taudiscr = 1.1; end
%  values of sigma in each layer
if ~any(strcmpi('sigma',nam)), opts.sigma = 0.5*1e2*ones(n,1); end
%  values of r_in for noreg
if ~any(strcmpi('r_in',nam)), opts.r_in = 0; end

truesol = opts.truesol;
L = get_l(n,opts.Lind);
metk = opts.metk;
show = opts.show;
showpar = opts.showpar;

complexdata = ~isreal(y);
if complexdata, mm = 2*mm; end

if opts.Lind==0		% set number of kappa values
	p = min(mm,n);
else
	p = size(L,1) - max(n-mm,0);
end

if p<=0
	error('Too few data to invert. You must increase the number of data values.')
end

kappavet = [1:p]';
rho = zeros(p,1);
eta = zeros(p,1);
X = zeros(n+4,p);
if ~isempty(truesol)
	err = zeros(p,1);
	maxmu = max(truesol);
end
if show
	if any(show==[1 2]), show = 10; end
	nrow = opts.rowcol(1);
	ncol = opts.rowcol(2);
	%fprintf('\n')
	figure(show)
	set(gcf,'NumberTitle','off','name','')
	clf
end

x = opts.muinit;
normx = norm(x);
normx0 = normx+eps;
opts.show = 0;			% show results & graphs in tsvdnewt
%opts.show = opts.show*4;
i = 0;
flag = 1;
while flag
	i = i+1;
	if normx < 1e8*normx0
		x0 = x;		% non e' detto che sia una buona idea
	else
		x0 = opts.muinit;
	end
	kappa = kappavet(i);
	tic
	[x,nit,tnflag,L,reserr] = tsvdnewt(func,y,x0,kappa,opts);
	tim = toc;

	X(1,i) = kappa;
	X(2,i) = nit;
	X(3,i) = tim;
	X(4,i) = tnflag;
	X(5:end,i) = x(:);
	
	rho(i) = reserr(end,1);		% residual
	eta(i) = norm(L*x);		% weighted norm
	normx = norm(x);
	
	if ~isempty(truesol)
		err(i) = norm(truesol-x);
	end
	
	flag = (i<p) && ((i==1) || (rho(i)<rho(i-1))) ...
		&& (eta(i)<1e8*eta(1)) && isfinite(rho(i)) && isfinite(eta(i));
	
	if show
		str = sprintf('kappa = %d,  n. iterations: %2d,  ', kappa, nit);
		str = [str sprintf('rel. residual = %.2e',rho(i)/norm(y))];
		if ~isempty(truesol)
			str = [str sprintf(',  error: %.2e', err(i))];
		end
		fdemprint( str, calledbyapp, textarea)
		z = [0;cumsum(d)];
		figure(show);
		subplot(nrow,ncol,mod(i-1,nrow*ncol)+1)
		if isempty(truesol)
			fdemplot(z,x,0)
		else
			fdemplot(z,[truesol x],0)
			ylim([0 2*maxmu])
		end
		set(gca,'fontsize',12)
		set(gca,'ButtonDownFcn',@(~,~)plotresults([1,2],z,x,truesol,kappa,func,opts.orientation,y,yrows,2))
		xlabel(sprintf('k=%d',kappa))
		if ~isempty(opts.xlim), xlim(opts.xlim), end
		if ~isempty(opts.ylim), ylim(opts.ylim), end
		drawnow
	end
	if app.stop, break, end

end

if show, set(gcf,'name','click on a solution to select it'), end

if i > 1	% if residuals are not decreasing, delete the last one
	if rho(i-1)-rho(i) > sqrt(eps)*rho(i-1)
		r = i;
	else
		r = i-1;
	end
else
	r = i;
end

if any(isnan(rho))
	inan = find(isnan(rho));
	r = inan(1)-1;
end

kappavet = kappavet(1:r);
rho = rho(1:r);
eta = eta(1:r);
X = X(:,1:r);

if ~isempty(truesol)
	err = err(1:r);
	[~,kopt] = min(err);
	x_o = X(5:end,kopt);
else
	err = nan;
end

if ischar(metk)
	metk = {metk}; 
else
	metk = metk(:);
end

cols = size(metk,1);
mu = zeros(n,cols);
kappa = zeros(1,cols);
fail = zeros(1,cols);
for j = 1:cols
	switch lower(metk{j})
        case 'noreg'
	    if (opts.r_in == 0) || (opts.r_in > r)
	        mu(:,j) = X(5:end,kappavet(r));
	        kappa(j) = kappavet(r);
	    else 
	        mu(:,j) = X(5:end, kappavet(opts.r_in));
	        kappa(j) = kappavet(opts.r_in);
	    end
	case 'optimal'
		if isempty(truesol), error('Missing true solution.'), end
		mu(:,j) = x_o;
		kappa(j) = kopt;
	case 'discrepancy'
		if isempty(opts.ds), error('Missing sdev of noise.'), end
		kdiscr = morozov(rho,opts.ds,'relative',norm(y),opts.taudiscr);
		if isempty(kdiscr)
			kdiscr = kappavet(end);
			warning('Discrepancy principle failed.')
			fail(j) = 1;
		end
		mu(:,j) = X(5:end,kdiscr);
		kappa(j) = kdiscr;
	case 'corner'
		kcorn = [];
		if length(rho)>=3
			if showpar
				figure(showpar+1)
				clf(showpar+1)
				kcorn = corner(rho,eta,showpar+1);
			else
				kcorn = corner(rho,eta);
			end
		end
		if isempty(kcorn)
			kcorn = kappavet(end);
			warning('Corner method failed, k=%d.',kcorn)
			fail(j) = 1;
		end
		mu(:,j) = X(5:end,kcorn);
		kappa(j) = kcorn;
	case 'resreg'
		if showpar
			figure(showpar+2)
			clf(showpar+2)
			krrf = rrfunction(rho,eta,[],[],[],showpar+2);
		else
			krrf = rrfunction(rho,eta,[],[],[]);
		end
		if isempty(krrf)
			krrf = kappavet(end);
			warning('ResReg method failed.')
			fail(j) = 1;
		end
		mu(:,j) = X(5:end,krrf);
		kappa(j) = krrf;
	case 'lresidual'
		if showpar
			figure(showpar+3)
			clf(showpar+3)
			klres = corner(rho,[1:size(rho,1)]',showpar+3);
		else
			klres = corner(rho,[1:size(rho,1)]');
		end
		if isempty(klres)
			klres = kappavet(end);
			warning('L-res method failed.')
			fail(j) = 1;
		end
		mu(:,j) = X(5:end,klres);
		kappa(j) = klres;
	case 'corner+1'
		if showpar
			figure(showpar+4)
			clf(showpar+4)
			kcorn = corner(rho,eta,showpar+4);
		else
			kcorn = corner(rho,eta);
		end
		if isempty(kcorn)
			kcorn = kappavet(end);
			warning('Corner method failed, k=%d.',kcorn)
			fail(j) = 1;
		elseif kcorn < p
			kcorn = kcorn + 1;
		end
		mu(:,j) = X(5:end,kcorn);
		kappa(j) = kcorn;
	case 'quasiopt'
		scarti = zeros(r-1,1);
		for i = 1:r-1
			%scarti(i) = norm(X(5:end,i)-X(5:end,i+1));
			scarti(i) = norm(L*(X(5:end,i)-X(5:end,i+1)));
		end
		[~,kqo] = min(scarti);
		if isempty(kqo)
			kqo = kappavet(end);
			warning('Q-opt method failed.')
			fail(j) = 1;
		end
		mu(:,j) = X(5:end,kqo);
		kappa(j) = kqo;
	case 'quasihyb'
		kcorn = corner(rho,eta);
		if isempty(kcorn)
			kcorn = kappavet(end);
			warning('Corner method failed.')
			fail(j) = 1;
		end
		%kqh = quasihybrid( X(5:end,:), kcorn);
		kqh = quasihybrid( L*X(5:end,:), kcorn);
		mu(:,j) = X(5:end,kqh);
		kappa(j) = kqh;
	case 'ltrian'
		kltrian = l_c_trian(rho,eta);
		if isempty(kltrian)
			kltrian = kappavet(end);
			warning('L-trian method failed.')
			fail(j) = 1;
		end
		mu(:,j) = X(5:end,kltrian);
		kappa(j) = kltrian;
	end
end

if show
	str = newline;
	for i = 1:numel(metk)
		str = [str sprintf('%12s: k = %2d', metk{i}, kappa(i))];
		if ~isempty(truesol)
			str = [str sprintf(', error = %.2e', norm(truesol-mu(:,i)))];
		end
		str = [str newline];
	end
	fdemprint( str, calledbyapp, textarea)
	plotresults([1 2],z,mu(:,1),truesol,kappa(1),func,opts.orientation,y,yrows,2)
end

