function plotresults(figs,z,sigma,truesol,kappa,func,orientation,y,yrows,whattocomp)
%PLOTRESULTS display intermediate results during iteration
%   the first figure displays the true solution, the second figure
%   represents the data fitting
%   plotresults(figs,z,sigma,truesol,kappa,func,orientation,y,yrows)
%   where
%      figs        - figures in which make the plot
%      z           - horizontal axis
%      sigma       - reconstructed conductivity in each layer (S/m)
%      truesol     - true solution in each layer (S/m)
%      kappa       - regularization parameters identified
%      func        - model to invert
%      orientation - orientation of the device
%      y           - data to fit
%      yrows       - rows to keep in model prediction
%      whattocomp  - 1 for sigma, 2 for mu

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised June 10, 2019

complexdata = ~isreal(y);
if complexdata, y = [real(y); imag(y)]; end
m = length(y);

figure(figs(1))
set(gcf,'name','Reconstruction')
clf
if isempty(truesol)
	hh = fdemplot( z, sigma, whattocomp);
	set(hh(1),'linewidth',2)
	legend( sprintf('k=%d',kappa))
	%title('Reconstruction of the solution')
else
	hh = fdemplot( z, [truesol sigma], whattocomp);
	set(hh(1),'linewidth',2)
	m1 = min(truesol);
	m2 = max(truesol);
	dd = (m2-m1)/5;
	ylim([m1-dd m2+dd])
	legend( 'solution', sprintf('k=%d',kappa))
	%title('Reconstruction of the solution')
end

if length(figs)==1, return, end

figure(figs(2))
set(gcf,'NumberTitle','off','name','Data Fitting')
%clf
yrec = func(sigma);
yrec = yrec(yrows);
if complexdata, yrec = [real(yrec); imag(yrec)]; end

if strcmpi(orientation(1:2),'vh')
	if complexdata
		y = y([1:m/4 m/2+1:3*m/4 m/4+1:m/2 3*m/4+1:m]);
		yrec = yrec([1:m/4 m/2+1:3*m/4 m/4+1:m/2 3*m/4+1:m]);
	end
	if mod(m,2), mm = m-1; else, mm = m; end	% WE SHOULD FIX THIS
	subplot(1,2,1)
	plot([1:mm/2]',y(1:mm/2),'o',[1:mm/2]',yrec(1:mm/2),'*')
	title('vertical','fontsize',11)
	axis tight
	ylabel('fields ratio H_s/H_p','fontsize',11);
	subplot(1,2,2)
	plot([mm/2+1:mm]',y(mm/2+1:mm),'o',[mm/2+1:mm]',yrec(mm/2+1:mm),'*')
	title('horizontal','fontsize',11)
	axis tight
	%ylabel('fields ratio H_s/H_p');
elseif strcmpi(orientation(1:2),'vp')
	if complexdata
		y = y([1:m/4 m/2+1:3*m/4 m/4+1:m/2 3*m/4+1:m]);
		yrec = yrec([1:m/4 m/2+1:3*m/4 m/4+1:m/2 3*m/4+1:m]);
	end
	if mod(m,2), mm = m-1; else, mm = m; end	% WE SHOULD FIX THIS
	subplot(1,2,1)
	plot([1:mm/2]',y(1:mm/2),'o',[1:mm/2]',yrec(1:mm/2),'*')
	title('vertical','fontsize',11)
	axis tight
	ylabel('fields ratio H_s/H_p','fontsize',11);
	subplot(1,2,2)
	plot([mm/2+1:mm]',y(mm/2+1:mm),'o',[mm/2+1:mm]',yrec(mm/2+1:mm),'*')
	title('perpendicular','fontsize',11)
	axis tight
else
	plot([1:m]',y,'o',[1:m]',yrec,'*')
	if strcmpi(orientation(1:2),'ve')
		title('vertical')
		ylabel('fields ratio H_s/H_p','fontsize',11);
	elseif strcmpi(orientation(1:2),'pe')
		title('perpendicular')
		ylabel('fields ratio H_s/H_p','fontsize',11);
	else
		title('horizontal')
		ylabel('fields ratio H_s/H_p','fontsize',11);
	end
end
xlabel(sprintf('k=%d',kappa),'fontsize',11)
legend('data','reconstructed')

