function [R0,RJsigma,RJmu] = reflfact(lambda,sigma,mu,d,omega,type)
%REFLFACT compute the reflection factor R(lambda)
%   [R0,RJsigma,RJmu]=reflfact(lambda,sigma,mu,d,omega,jtype)
%   where
%      lambda - array of evaluation points
%      sigma  - array of conductivities per layer (S/m)
%      mu     - array of magnetic permeabilities per layer (H/m)
%      d      - array of layers thickness (m)
%      omega  - array of angular frequencies (rad/s) (=2*pi*f, f in Hz)
%      jtype  - Jacobian type: 'sigma', 'mu', 'both'
%               (used only when there are two output parameters)
%   and
%      R0      - array containing R0(lambda) (complex valued)
%                it is a matrix with q=length(lambda) rows and
%                nfreq=length(omega) columns
%      RJsigma - gradient w.r.to sigma_j
%                it is a 3D array of size q X n=length(sigma) X nfreq
%      RJmu    - gradient w.r.to mu_j
%                it is a 3D array of size q X n=length(mu) X nfreq

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised June 10, 2019

compjack = false;
compsigma = false;
compmu = false;
if nargout > 1
	compjack = true;
	if nargout>2 
		compsigma = true;
		compmu = true;
	elseif nargin>5
		switch lower(type(1))
		case 's'
			compsigma = true;
		case 'm'
			compmu = true;
		otherwise
			error('The type parameter is wrong.')
		end
	else
		error('The type parameter is missing.')
	end
end

lambda = lambda(:);
sigma = sigma(:);
mu = mu(:);
omega = omega(:);
n = size(sigma,1);
q = size(lambda,1);
nfreq = size(omega,1);
mu0 = pi*4e-7;

%
% Computing constants
%
den0 = complex(0,mu0*omega);
densig = mu*omega.';				% densig(k,l) = mu(k)*omega(l)
frac = complex(0,-1./densig);			% frac(k,l) = 1/(1i*mu(k)*omega(l))
densig = repmat(sigma,1,nfreq).*densig;		% densig(k,l) = mu(k)*sigma(k)*omega(l)

%
% Computing Y matrix
%
lam2 = repmat(lambda.^2,1,nfreq);
u = sqrt(complex(lam2,repmat(densig(n,:),q,1)));	% u_nl(lambda)
Y = u.*repmat(frac(n,:),q,1);				% Y_nl(lambda)=N_nl(lambda)

if compjack
	arg_sup = 3e2;					% found experimentally
	fact = 0.5./u;
	if compsigma
		RJsigma = zeros(q,n,nfreq);
		Ds = zeros(q,n,nfreq);
		Ds(:,n,:) = fact;
	end
	if compmu
		RJmu = zeros(q,n,nfreq);
		sigfmu = sigma./mu;
		Dm = zeros(q,n,nfreq);
		Dm(:,n,:) = sigfmu(n).*fact - Y/mu(n);
	end
end

for k = n-1:-1:1
	u = sqrt(complex(lam2,repmat(densig(k,:),q,1)));	% u_kl(lambda)
	N = u.*repmat(frac(k,:),q,1);				% N_kl(lambda)
	arg = u*d(k);
	tan_ip = tanh(arg);
	Aden = N+Y.*tan_ip;
	A = (Y+N.*tan_ip)./Aden;
	
	if compjack
		N2 = N.^2;
		cos_ip2 = cosh(arg);
		for l = 1:nfreq
			vi = real(arg(:,l)) < arg_sup;
			nvi = sum(vi);
			cos_ip2(:,l) = [1./cos_ip2(vi,l).^2;zeros(q-nvi,1)];
		end
		B = 1./Aden.^2 .* cos_ip2;
		%N2B = N2.*B;
		N2B = permute(repmat(N2.*B,[1,1,n-k]),[1 3 2]);
		rfrack = repmat(frac(k,:),q,1);
		if compsigma
			% which is better?
			% this seems faster for large nfreq
			%for j = n:-1:k+1
			%	Ds(:,j,:) = N2B .* squeeze(Ds(:,j,:));
			%end
			% this seems faster for small nfreq
			Ds(:,k+1:n,:) = N2B .* Ds(:,k+1:n,:);
			par = N2*d(k) - Y.*(Y*d(k) + rfrack);
			Ds(:,k,:) = 0.5*(A./u + B.*par);
		end
		if compmu
			Dm(:,k+1:n,:) = N2B .* Dm(:,k+1:n,:);
			Dm(:,k,:) = 0.5*d(k)*sigfmu(k)*B.*(N2-Y.^2) ...
				-rfrack/mu(k).*(A-B.*N.*Y).*(u-sigma(k)/2./N);
		end
	end
	Y = N.*A;		% Y_kl(lambda)
end

%
% Computing the reflection factor
%
N0 = lambda * (1./den0.');	% computing N_0 value
R0 = (N0-Y)./(N0+Y);		% computing R_0 value

if compjack
	for j = 1:n
		fact = -2*lambda*den0.' ./ ...
			(lambda*ones(1,nfreq) + Y(:,:).*(ones(q,1)*den0.')).^2;
		if compsigma
			% which is better?
			%for l = 1:nfreq
			%	RJsigma(:,j,l) = fact .* Ds(:,j,l);
			%end
			RJsigma(:,j,:) = fact .* squeeze(Ds(:,j,:));
		end
		if compmu
			RJmu(:,j,:) = fact .* squeeze(Dm(:,j,:));
		end
	end
end
if nargout==2 && compmu
	RJsigma = RJmu;
end

