function [A,B,z] = sensitivityfunc(sigma,mu,z,lastzplot,h,d,R,omega,HVorient,modeltype,fff)
%SENSITIVITYFUNC compute the sensitivity function
%   sensitivityfunc(sigma,mu,z,lastz,h,d,R,omega,HVorient,modeltype)
%   where
%       sigma       - array of conductivities per layer (S/m)
%       mu          - array of magnetic permeabilities per layer (H/m)
%       z           - array of depths (m)
%	    lastz	    - max z displayed on graphs
%       h           - array of heights above ground (m)
%       d           - array of layers thickness (m)
%       R           - array of intercoil (IC) spacings (m)
%       omega       - array of angular frequencies (rad/s) (=2*pi*f, f in Hz)
%       HVorient    - instrument orientation: 'vertical', 'horizontal', 
%				'perpendicular', 'vh', 'vp'
%       modeltype   - kind of instruments readings: 1 aconduct, 2 quadracomp,
%				3 inphase, 4 complexquad, 5 complexacon
%   and
%       A   - array of sensitivities for the electrical conductivity
%       B   - array of sensitivities for the magnetic permeability
%       z   - array of depths (m)
%
%   G.P. Deidda, P. Díaz de Alba, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised January 26, 2023

if nargin<11, fff = 0; end
legendavail = 1;
pos = 'southeast';
freq = omega/(2*pi);
lastz = z(end);
n = length(z);

switch modeltype		% define the model to invert
	case 1
		func = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 2
		func = @(sigma,mu) quadracomp(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) quadracomp(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 3
		func = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 4
		func = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 5
		funcinphase = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcacond = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcinphasemu = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'mu');
		funcacondmu = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'mu');
	%case 5, func = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'sigma');
end

if modeltype == 5
	[~,Js1] = funcinphase(sigma,mu);
	[~,Js2] = funcacond(sigma,mu);
	A = Js1 + Js2*1i;
	[~,Jm1] = funcinphasemu(sigma,mu);
	[~,Jm2] = funcacondmu(sigma,mu);
	B = Jm1 + Jm2*1i;
else
	[~,A] = func(sigma,mu);
	[~,B] = funcmu(sigma,mu);
end
xlab = 'Depth (m)';
legendsens = strcat('\rho=',strtrim(cellstr(num2str(R(:)))),',freq=',strtrim(cellstr(num2str(freq(:)))),',h=',strtrim(cellstr(num2str(h(:)))));
if modeltype == 1
	A = 1e3.*A/(lastz/(n-1));
	B = 1e-3.*B/(lastz/(n-1));
	ylab1 = 'Sensitivity [(mS/m)/(S/m)]';
	ylab2 = '';
	ylab3 = 'Sensitivity [(mS/m)/(\muH/m)]';
	ylab4 = '';
	%legpos = 'southeast';
elseif modeltype == 2
	A = 1e6.*A/(lastz/(n-1));
	ylab1 = 'Sensitivity [ppm/(S/m)]';
	ylab2 = '';
	ylab3 = 'Sensitivity [ppm/(\muH/m)]';
	ylab4 = '';
	%legpos = 'southeast';
elseif modeltype == 3
	A = 1e6.*A/(lastz/(n-1));
	ylab1 = 'Sensitivity [ppm/(S/m)]';
	ylab2 = '';
	ylab3 = 'Sensitivity [ppm/(\muH/m)]';
	ylab4 = '';
	%legpos = 'southeast';
elseif modeltype == 4
	A = 1e6.*A/(lastz/(n-1));
	ylab1 = 'Sensitivity [ppm/(S/m)]';
	ylab2 = 'Sensitivity [ppm/(S/m)]';
	ylab3 = 'Sensitivity [ppm/(\muH/m)]';
	ylab4 = 'Sensitivity [ppm/(\muH/m)]';
	%legpos = 'southeast';
elseif modeltype == 5
	A = 1e3.*A/(lastz/(n-1));
	B = 1e-3.*B/(lastz/(n-1));
	ylab1 = 'Sensitivity [ppt/(S/m)]';
	ylab2 = 'Sensitivity [(mS/m)/(S/m)]';
	ylab3 = 'Sensitivity [ppt/(\muH/m)]';
	ylab4 = 'Sensitivity [(mS/m)/(\muH/m)]';
	%legpos = 'southeast';
end

if fff
	plotsensfunc(fff,A,B,z,lastzplot,HVorient,xlab,ylab1,ylab2,ylab3,ylab4,legendavail,pos,legendsens,modeltype)
end

A = A(:,1:end-1).';
B = B(:,1:end-1).';
z = z(1:end-1);

