function [x,k,flag,L,out] = tsvdnewt(funj,y,x0,kappa,opts)
%TSVDNEWT Gauss-Newton method regularized by T(G)SVD
%   [x,k,flag,L,reserr]=tsvdnewt(funj,y,x0,kappa,opts)
%   where
%      funj  - function to compute the function and the Jacobian matrix
%      y     - data to fit
%      x0    - initial value
%      kappa - regularization parameter
%      opts  - options (struct, every options has a default)
%              Lind: index of regularization matrix (k-th derivative)
%              tau: stop tolerance
%              nmax: max number of iterations
%              damped: 1 damped method, 0 undamped
%              dampos: 1 positive solution, 0 unconstrained
%              metjac: method to compute/approximate the Jacobian (0 Newton,
%                      1 Broyden, 2 updated Broyden, 4 finite differences)
%              kbroyden: interval for Broyden's Jacobian updates
%              show: >0 show information on iterations
%              truesol: exact solution, if available (to compute errors)
%              mnflag: 1-2-3-4 minimal-norm solution, 0 standard solution
%              xbar: a priori estimate for solution (only if mnflag>0)
%              betamin: minimum value for bet
%              eta3: tolerance for residual increase
%              eta4: tolerance for residual increase
%              kres: number of residuals to determine eta4
%   and
%      x      - solution
%      k      - number of iterations
%      flag   - stop condition
%      L      - regularization matrix
%      reserr - residuals (and errors, if truesol is given)

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, F. Pes, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised October 21, 2022

nam = fieldnames(opts);
% weight for in-phase component, when the signal is complex
if ~any(strcmpi('beta',nam)), opts.beta = 1; end

sqbet = sqrt(opts.beta);
complexdata = ~isreal(y);
if complexdata, y = [sqbet*real(y); imag(y)]; end

m = length(y);
n = length(x0);

% rows to keep in model prediction
if ~any(strcmpi('yrows',nam)), opts.yrows = [1:m]'; end
% index of regularization matrix (index of derivative)
if ~any(strcmpi('Lind',nam)), opts.Lind = 1; end
% type of regularization matrix (0 derivative, 1 MGS)
if ~any(strcmpi('Ltype',nam)), opts.Ltype = 0; end
% method for computing the Jacobian
if ~any(strcmpi('metjac',nam)), opts.metjac = 0; end
% number of Broyden updates before recomputing the Jacobian when metjac=2
if ~any(strcmpi('kbroyden',nam)), opts.kbroyden = 10; end
% stopping tolerance
if ~any(strcmpi('tau',nam)), opts.tau = 1e-4; end
% maximum number of iterations for Gauss-Newton method
if ~any(strcmpi('nmax',nam)), opts.nmax = 100; end
%  damped method
if ~any(strcmpi('damped',nam)), opts.damped = 1; end
%  damping parameter is used to keep the solution positive
if ~any(strcmpi('dampos',nam)), opts.dampos = 1; end
%  sigmam parameter for MGS regularizer
if ~any(strcmpi('sigmam',nam)), opts.sigmam = 1e-4; end
%  epsilon parameter for MGS regularizer
if ~any(strcmpi('epsilon',nam)), opts.epsilon = 1e-4; end
%  show intermediate results & graphs
if ~any(strcmpi('show',nam)), opts.show = 0; end
%  if true solution is available, compute errors
if ~any(strcmpi('truesol',nam)) || isempty(opts.truesol)
	keeperr = 0; opts.truesol = [];
else
	keeperr = 1;
end
%  minimal norm solution
if ~any(strcmpi('mnflag',nam)), opts.mnflag = 4; end
%  a priori estimate for solution (only if mnflag>0)
if ~any(strcmpi('xbar',nam)), opts.xbar = zeros(size(x0)); end
% minimum value for bet
if ~any(strcmpi('betamin',nam)), opts.betamin = 1e-9; end
% tolerance for residual increase
if ~any(strcmpi('eta3',nam)), opts.eta3 = 8; end
% tolerance for residual increase
if ~any(strcmpi('eta4',nam)), opts.eta4 = 1/8; end
% number of residuals to determine eta4
if ~any(strcmpi('kres',nam)), opts.kres = 5; end

if opts.mnflag==0 && ~(isequal(opts.xbar,zeros(size(x0))))
	warning('xbar is useful only if mnflag>0.')
end

if ((opts.mnflag==5) || (opts.mnflag==6)) && (opts.damped==1)
	opts.damped=0;
	warning('using undamped method')
end

eta3 = opts.eta3;
eta4 = opts.eta4;

yrows = opts.yrows;
tsol = opts.truesol;
L = get_l(n,opts.Lind);
p = size(L,1);
mmn = min(m,n);
d = n-p;

% MGS regularizing term
if opts.Ltype == 1
	sigmam = opts.sigmam;
	epsilon = opts.epsilon;
	if max(size(sigmam)) == 1, sigmam = sigmam*ones(p,1); end
	L0 = L;
	% the first solution is constant, it is probably better
	% not to evaluate mgsreg on it
	if kappa>1		% non e' detto che sia una buona idea
		L = mgsreg(x0,L0,sigmam,epsilon);
	end
end

minpowAG = -15;		% alpha > 2^minpowAG for Armijo-Goldstein

res = zeros(opts.nmax+1,1);
if keeperr, err = zeros(opts.nmax,1); end

k = 0;
x = x0;
normx0 = norm(x0);
bet = 1;

flag = 1;
flagalpha = 1;
while flag
	k = k + 1;

	switch opts.metjac
		case 0		% Newton
			[fx,J] = funj(x);
			if complexdata
				fx = [sqbet*real(fx); imag(fx)];
				J = [sqbet*real(J); imag(J)];
			end
		case 1		% Broyden update
			if k == 1
				[fx,J] = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
					J = [sqbet*real(J); imag(J)];
				end
			else
				fold = fx;
				fx = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
				end
				v = fx - fold;
				J = J + ((v - J*q)*q')./(q'*q);
			end
		case 2		% Broyden update every 10 iterations
			if mod(k,opts.kbroyden) == 1
				[fx,J] = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
					J = [sqbet*real(J); imag(J)];
				end
			else
				fold = fx;
				fx = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
				end
				v = fx - fold;
				J = J + ((v - J*q)*q')./(q'*q);
			end
		case 3		% Corde (constant J for 10 iterations)
			if mod(k,10) == 1
				[fx,J] = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
					J = [sqbet*real(J); imag(J)];
				end
			end
		case 4		% finite differences
			dx = 1e-6;
			fx = funj(x);
			J = jack(funj,x,dx);
			if complexdata
				fx = [sqbet*real(fx); imag(fx)];
				J = [sqbet*real(J); imag(J)];
			end
		otherwise
			error('Unknown method.')
	end

	fx = fx(yrows);
	J = J(yrows,:);
	b = y-fx;	% -(fx-y)
	res(k) = norm(b);
	x0 = x;
	if opts.Lind==0
		[U,S,V] = svd(J);
		if mmn == 1
			s = S(1,1);
		else
			s = diag(S);
		end
		if kappa<n
			V2 = V(:,kappa+1:n);
		else
			V2 = 0;
		end
		U = U(:,1:kappa);
		V = V(:,1:kappa);
		s = s(1:kappa);
	else
		L = full(L);
		[U,~,WI,C,~] = gsvd(J,L);
		WI = WI';
		[LWI, UWI, PWI] = lu(WI);
		if mmn == 1
			c = C(1,n);
		else
			c = diag(C(1:mmn,n-mmn+1:n));
		end
		c = c(mmn-d-kappa+1:mmn);	% includes the ones
		U = U(:,mmn-d-kappa+1:mmn);
	end

	if opts.Lind==0
		yy = U'*b;
		yy = yy./s;
		q = V*yy;
		if opts.mnflag
			tk = V2*(V2'*(x0-opts.xbar));
		end
	else
		yy = U'*b;
		yy = yy./c;
		if kappa<p
			z = WI(1:p-kappa,:)*(x0-opts.xbar);
			rhs = [[z;zeros(n-p+kappa,1)] [zeros(p-kappa,1);yy]];
			sol = LWI \ (PWI*rhs);
			sol = UWI \ sol;
			tk = sol(:,1);
			q = sol(:,2);
		else
			sol = LWI \ (PWI*yy);
			q = UWI \ sol;
			tk = 0;
		end
	end

	if opts.mnflag == 2
		q = q - tk;
	end

	if opts.damped
		% Armijo-Goldstein principle
		alpha = 2;
		flagd = 1;
		if opts.show, fprintf('step %d, alpha = ', k), end
		njq2 = norm(J*q)^2;
		while flagd
			alpha = alpha / 2;
			x1 = x0+alpha*q;
			fx = funj(x1);
			if complexdata, fx = [sqbet*real(fx); imag(fx)]; end
			res1 = norm(y-fx(yrows));
			flagd = (res(k)^2-res1^2) < alpha/2*njq2;
			if opts.dampos, flagd = flagd || any(x1<0); end
			flagalpha = norm(alpha*q)>pow2(minpowAG);
			flagd = flagd && flagalpha;
			if opts.show, fprintf('%.1e, ',alpha), end
		end
		if opts.show, fprintf('\n'), end
		%x1 = x0 + alpha*q;	% already computed
	else
		x1 = x0 + q;
	end

	switch opts.mnflag	% minimal-norm and step lenght bet
		case 0		% no minimal-norm
			x2 = x1;
		case 1		% bet=1
			x2 = x1 - tk;
		case 2		% 1 parameter, bet=alpha
			x2 = x1;
		case 3		% 2 parameters, constant eta3
			rho1 = norm(funj(x1)-y);
			if bet<1, bet = bet*2; end
			x2 = x1 - bet*tk;
			rho2 = norm(funj(x2)-y);
			while (rho2>(1+eta3)*(rho1+eps)) && (bet>opts.betamin)
				bet = bet/2;
				x2 = x1 - bet*tk;
				rho2 = norm(funj(x2)-y);
			end
		case 4		% 2 parameters, adaptive eta4
			rho1 = norm(funj(x1)-y);
			if bet<1, bet = bet*2; end
			x2 = x1 - bet*tk;
			rho2 = norm(funj(x2)-y);
			if k>=opts.kres
				pol = polyfit([1:opts.kres]',...
					log10(res(k-opts.kres+1:k)),1);
				if (pol(1)>-1e-2)
					if eta4<1e4, eta4 = 2*eta4; end
				elseif (pol(1)<-1/2)
					eta4 = eta4/2;
				end
			end
			while (rho2>rho1+(rho1+eps)^eta4) && (bet>opts.betamin)
				bet = bet/2;
				x2 = x1 - bet*tk;
				rho2 = norm(funj(x2)-y);
			end
		case 5		% CKB1
			alpha = 0.5^k;
			x2 = x1 - alpha*tk;
		case 6		% CKB2
			alpha = 0.5^(2^(k-1));
			x2 = x1 - alpha*tk;
		otherwise
			error('Wrong mnflag value.')
	end

	x = x2;

	% MGS regularizing term
	if opts.Ltype == 1
		L = mgsreg(x,L0,sigmam,epsilon);
	end

	if keeperr, err(k) = norm(tsol-x); end

	if opts.show
		figure(opts.show)
		if keeperr
			plot([tsol x],'o-')
		else
			plot(x,'o-')
		end
		title(sprintf('TSVD-Newton with kappa = %d',kappa))
	end

	diff = norm(x-x0);
	normx = norm(x);
	flag1 = (diff > opts.tau*max(1,normx));
	flag2 = (normx < 1e8*normx0);
	flag3 = (k<=opts.nmax);
	flag = flag1 && flag2 && flag3 && flagalpha;
end
fx = funj(x);
if complexdata, fx = [sqbet*real(fx); imag(fx)]; end
res(k+1) = norm(fx(yrows)-y);
flag = flag1+10*flag2+100*flag3+1000*flagalpha;

if keeperr
	out = [res(2:k+1) err(1:k)];
else
	out = [res(2:k+1)];
end

